%{
/*****************************************************************************
 *            ASM: a minimal assembler for educational purpose               *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2010 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
#include <string.h>	/* for strcmp, strdup & friends */
#include <stdio.h>	/* for strcmp, strdup & friends */
#include <stdlib.h>	/* for atoi() */

#include "parser.h"	/* (TK_) symbols from parser.y */

extern int line_number;	/* defined in parser.y */

%}

%x comment
%x str

%%
[ \t\r]			break;		/* ignore white space */
"\n" 			++line_number; 

"["                     return TK_LBRAC;
"]"                     return TK_RBRAC;
"("                     return TK_LPAR;
")"                     return TK_RPAR;
","                     return TK_COMMA;
":"                     return TK_COLON;

"+"                     return TK_PLUS;
"-"                     return TK_MINUS;

"add"                   return TK_ADD;
"sub"                   return TK_SUB;
"ldi"                   return TK_LDI;
"ld"                    return TK_LD;
"st"                    return TK_ST;
"ble"                   return TK_BEQ;
"j"                     return TK_J;
"ja"                    return TK_JA;

"r0"                    {
                          yylval.integer = 0; 
                          return TK_REG;
                        }
"r1"                    {
                          yylval.integer = 1; 
                          return TK_REG;
                        }
"r2"                    {
                          yylval.integer = 2; 
                          return TK_REG;
                        }
"r3"                    {
                          yylval.integer = 3; 
                          return TK_REG;
                        }
"r4"                    {
                          yylval.integer = 4; 
                          return TK_REG;
                        }
"r5"                    {
                          yylval.integer = 5; 
                          return TK_REG;
                        }
"r6"                    {
                          yylval.integer = 6; 
                          return TK_REG;
                        }
"r7"                    {
                          yylval.integer = 7; 
                          return TK_REG;
                        }
"r"[0-9]+               {
                          printf("Error: undefined register %s line %d\n"
                                 ,yytext,line_number);
                          exit(1);
                        }
                         

"lo"                    return TK_LO;
"hi"                    return TK_HI;

"data"                  return TK_DATA;

"//"                    BEGIN(comment); //
<comment>[^\n]          break;          // Comments handling
<comment>\n             BEGIN(INITIAL); //

\"                      BEGIN(str); // Strings handling        //
<str>[^\"]*             {                                      //
                          yylval.id_string = strdup(yytext);   //
                          return TK_STRING;                    //
                        }                                      //
<str>\"                 BEGIN(INITIAL);                        //

\'.\'                   {
                          yylval.integer = (char) *(yytext+1); 
                          return TK_CAR; 
                        }                          

[0-9]+                  { 
                          yylval.integer = (unsigned short int)atoi(yytext); 
                          return TK_INT; 
                        }

[A-Za-z_]([A-Za-z0-9_])* {
                          yylval.id_string = strdup(yytext);
                          //cout << yylval.id_string << endl;
                          return TK_ID;
                        }

.			{
			  fprintf(stderr,
				"Illegal character with code %d on"
				"line #%d\n",
				*yytext, line_number);
			  exit(1);
			}
